import numpy as np


class Probability_Moment:
    def __new__(cls, *args, **kwargs):
        instance = super(Probability_Moment, cls).__new__(cls)
        return instance

    def __init__(self, Y, f):
        self.Y = Y #Visto em Benasciutti Tovo sobre one-sided spectral density
        self.f = f
        self.df = f[1] - f[0]

        '''
            If calculate m_i through integral of (G_yy(f)*(2*pi*f)^i)df, 
            the result of sqrt(m_2) is close to sigma_Xdot 
            and sqrt(m_4) is close to sigma_double dot, but the error is bigger than obtained
            without this (2*pi)
        '''

    def moment0(self):

        m0 = 0

        # Por Dirlik, mi = (1/2pi) integral (w^i)*G(w)dw de -inf até inf

        for i in range(0, len(self.f)):
            if self.f[i] >= 0:
                m0 += self.Y[i]*self.df

        return m0

    def moment075(self):
        m75 = 0

        for i in range(0, len(self.f)):
            if self.f[i] >= 0:
                m75 += np.abs(self.Y[i] * (2*np.pi*self.f[i])**0.75)*self.df

        return m75

    def moment1(self):
        m1 = 0

        for i in range(0, len(self.f)):
            if self.f[i] >= 0:
                m1 += np.abs(self.Y[i] * 2*np.pi*self.f[i])*self.df

        return m1

    def moment15(self):
        m15 = 0

        for i in range(0, len(self.f)):
            if self.f[i] >= 0:
                m15 += np.abs(self.Y[i] * (2*np.pi*self.f[i])**1.5)*self.df

        return m15

    def moment2(self):
        m2 = 0


        for i in range(0, len(self.f)):
            if self.f[i] >= 0:
                m2 += np.abs(self.Y[i] * (2*np.pi*self.f[i]) ** 2)*self.df

        return m2

    def moment4(self):
        m4 = 0

        for i in range(0, len(self.f)):
            if self.f[i] >= 0:
                m4 += np.abs(self.Y[i] * (2*np.pi*self.f[i]) ** 4)*self.df

        return m4
    
    def E0(self):
        return np.sqrt(self.moment2()/self.moment0())/(2*np.pi)
    
    def EP(self):
        return np.sqrt(self.moment4()/self.moment2())/(2*np.pi)
    
    def alpha1(self):
        return self.moment1()/np.sqrt(self.moment0()*self.moment2())

    def alpha2(self):
        return self.moment2()/np.sqrt(self.moment0()*self.moment4())
    
    def alpha075(self):
        return self.moment075()/np.sqrt(self.moment0()*self.moment15())




import rainflow
import matplotlib.pyplot as plt
import numpy as np
from . import prob_moment

class rainflowD:


    def __init__(self, C, k, y, x):
        self.C = C
        self.k = k
        self.b = -1/k
        self.A = C**(1/k)
        self.y = y
        self.x = x

    def rainflow_histogram(self):
        cc = rainflow.count_cycles(self.y, nbins=50)

        r = []
        n = []
        for i in range(len(cc)):
            r.append(cc[i][0]/2)
            n.append(cc[i][1])

        '''
        nn = np.zeros(len(cc))
        rng = np.zeros(len(cc))
        nnn = np.zeros(len(cc))
        rngrou = np.zeros(len(cc))
    
        for j in range(len(cc)):
            nn[j] = cc[j][1]
            rng[j] = cc[j][0]
            for i in range(1, len(cc)):
                if round(rng[i - 1], 4) == round(rng[j - 1], 4):
                    rngrou[j] = round(rng[i], 4)
                    nnn[j] += nn[i]
        

        vl = []
        pk = []
        cnt = []
        sa = []
        sm = []
        j = 0
        with open('rainflow.txt', 'w') as file:
            file.write("      Valley                Peak         Count           Sa                 Sm\n")
            for valley, peak, count in rainflow.extract_cycles(self.y): 
                #print(valley, peak, count)
                vl.append(valley)
                pk.append(peak)
                cnt.append(count)
                sa.append((pk[j]-vl[j])/2) 
                sm.append((pk[j]+vl[j])/2)
                
                file.write(str(vl[j])+"    ")
                file.write(str(pk[j])+"   ")
                file.write(str(cnt[j])+"    ")
                file.write(str(sa[j])+"    ")
                file.write(str(sm[j])+" ")
                file.write("\n")
                
                j+=1

        DRF = 0
        Nf = []
        s = []
        n = []
        sigmaf = self.A/(2**self.b)
        for i in range(len(sa)):
            s.append(sa[i]/(1-(sm[i]/sigmaf)))
            n.append(cnt[i])
        '''

        # Begins here!!!!
        tns = sum(n)

        rangemax = max(r) - min(r)

        nclass = 50

        rangesm = []
        auxs = []
        smean = []

        nmatrix = []
        S = []
        sigmaf = self.A/(2**self.b)

        for j in range(nclass):
            sumn = 0
            for i in range(len(r)):
                if(r[i] >= min(r) + (rangemax/nclass)*(j)  and r[i] < min(r) + (rangemax/nclass)*(j+1)):
                    auxs.append(r[i])
                    sumn += n[i]

                if (r[i] == max(r) and j == nclass-1):
                    auxs.append(r[i])
                    sumn +=n[i]
            
            nmatrix.append(sumn)

            smean.append((min(r) + (rangemax/nclass)*(j) + min(r) + (rangemax/nclass)*(j+1))/2)
            rangesm.append(auxs[:])
            auxs.clear()

            S.append(smean[j]/(1-(self.y.mean()/sigmaf)))



        bw = (min(r) + (rangemax/nclass)*(j+1)) - (min(r) + (rangemax/nclass)*(j))

        p = []
        summ = 0

        for i in range(len(nmatrix)):
            p.append((nmatrix[i]/tns)/bw)
        
        #Finish here!!!!

        return S, nmatrix

    def CumuCycles(self):
        S, n = self.rainflow_histogram()
        CumuC = np.zeros(len(S))

        for i in range(len(S)):
            for j in range(i, len(S)):
                CumuC[i] += n[j]
        
        return CumuC, S


    def Damage(self):
        '''
        cc = rainflow.count_cycles(self.y)

        vl = []
        pk = []
        cnt = []
        sa = []
        sm = []
        j = 0
        
            
        for valley, peak, count in rainflow.extract_cycles(self.y): 
            #print(valley, peak, count)
            vl.append(valley)
            pk.append(peak)
            cnt.append(count)
            sa.append((pk[j]-vl[j])/2) 
            sm.append((pk[j]+vl[j])/2)
                
            j+=1

        DRF = 0
        Nf = []
        s = []
        n = []
        sigmaf = self.A/(2**self.b)
        for i in range(len(sa)):
            s.append(sa[i]/(1-(sm[i]/sigmaf)))
            n.append(cnt[i])
            Nf.append((s[i]/self.A)**(1/self.b))
            DRF += n[i] / Nf[i]


            DRF = DRF/max(self.x)
        '''

        S, n = self.rainflow_histogram()
        DRF = 0
        for i in range(1,len(n)):
            Nf = self.C/(S[i]**self.k)
            DRF += n[i]/Nf 
        
        return DRF/max(self.x)

    def Lifes(self):
        TRFs =  1/self.Damage()
        return TRFs
    
    def Lifeh(self):
        TRFh = self.Lifes()/3600
        return TRFh
    
    def Life(self):
        TRF = self.Lifes()/max(self.x)
        return TRF



import numpy as np
import math
from . import prob_moment, Rainflow

class ZB:
    def __new__(cls, *args, **kwargs):
        instance = super(ZB, cls).__new__(cls)
        return instance

    def __init__(self, k, C, Y, f, xf, s):
        self.Y = Y
        self.f = f
        moments = prob_moment.Probability_Moment(self.Y, self.f)
        self.m0 = moments.moment0()
        self.m1 = moments.moment1()
        self.m2 = moments.moment2()
        self.m4 = moments.moment4()
        self.m75 = moments.moment075()
        self.m15 = moments.moment15()
        self.alpha2 = moments.alpha2()
        self.k = k
        self.C = C
        self.EP = moments.EP()
        self.xf = xf
        self.s = s

    def beta(self):
        if self.alpha2<0.9:
            beta = 1.1
        else:
            beta = 1.1+9*(self.alpha2-0.9)
        return beta


    def PDF(self):
        z = self.s / (np.sqrt(self.m0))
        beta = self.beta()
        EGFbeta = math.gamma(1 + 1/beta)
        EGF3beta = math.gamma(1+3/beta)

        if self.k>3:
            alpha = 8 - 7*self.alpha2 #for ZB1
        elif self.k<=3:
            
            alpha075 = self.m75/(np.sqrt(self.m0*self.m15))

            if alpha075 < 0.5:
                roZB = 0.28
            else:
                roZB = -0.4154 + 1.392*alpha075
            d = 10
            for i in range(10):
                d = d - (EGF3beta*(1-self.alpha2**2)*d**3+3*EGFbeta*(roZB*self.alpha2-1)*d+3*np.sqrt(np.pi/2)*self.alpha2*(1-roZB))\
                    /(3*EGF3beta*(1-self.alpha2**2)*d**2+3*EGFbeta*(roZB*self.alpha2-1))
                alpha = d**(-beta)
        

        w = (1 - self.alpha2)/(1 - math.sqrt(2/math.pi)*EGFbeta*alpha**(-1/beta))

        ps = (w*alpha*beta*z**(beta - 1)*np.exp(-alpha*z**beta) + (1-w)*z*np.exp((-z**2)/2))/(np.sqrt(self.m0)) 

        return ps
    
    def counting_cycles(self):
        ps = self.PDF()
        ds = self.s[1] - self.s[0]
        ns = ps*ds*self.EP*self.xf

        return ns
    
    def loading_spectrum(self):
        Cs = np.zeros(len(self.s))
        ns = self.counting_cycles()

        for i in range(len(self.s)):
            for j in range(i, len(self.s)):
                Cs[i] += ns[j]
        
        return Cs

    def Damage(self):
        ps = self.PDF()
        ds = self.s[1] - self.s[0]
        DZB = 0
        for i in range(1,len(ps)):
            DZB += abs(self.EP*(self.C**(-1))*(self.s[i]**self.k)*ps[i]*ds)

        return DZB

    def Lifes(self):
        TZB = 1 / self.Damage()
        return TZB

    def Lifeh(self):
        TZBh = self.Lifes()/(3600)
        return TZBh

    def Life(self):
        TZB = self.Lifes()/self.xf
        return TZB

    def relative_error(self, y, x, method="Rainflow", experimental_value=None, type='cycles'):
        if type=="cycles":
            ZB_value = self.Life()
            RF_value = Rainflow.rainflowD(self.C, self.k, y, x).Life()
        elif type=="damage":
            ZB_value = self.Damage()
            RF_value = Rainflow.rainflowD(self.C, self.k, y, x).Damage()
        elif type!="cycles" and type!="damage":
            raise UnboundLocalError("Invalid type. Try 'cycles' or 'damage'")
        
        if(method == "Rainflow"):
            err = (ZB_value - RF_value)/RF_value
        elif(method == "Experimental" and experimental_value != None):
            EX_value = experimental_value
            err = (ZB_value - EX_value)/EX_value
        elif(method == "Experimental" and experimental_value == None):
            raise UnboundLocalError("Dexperimental must be different from None for method 'Experimental'")
        elif(method != "Experimental" and method != "Rainflow"):
            raise UnboundLocalError("Invalid Method. Try method='Rainflow' or method='Experimental'")

        return err



import numpy as np
import math
from . import prob_moment, Rainflow

class DK:
    def __new__(cls, *args, **kwargs):
        instance = super(DK, cls).__new__(cls)
        return instance

    def __init__(self, k, C, Y, f, xf, s):
        self.Y = Y
        self.f = f
        moments = prob_moment.Probability_Moment(self.Y, self.f)
        self.m0 = moments.moment0()
        self.m1 = moments.moment1()
        self.m2 = moments.moment2()
        self.m4 = moments.moment4()
        if round(moments.alpha2(), 4) != 1.0:
            self.alpha2 = moments.alpha2()
        else:
            self.alpha2 = 0.99
        self.k = k
        self.C = C
        self.EP = moments.EP()
        self.xf = xf
        self.s = s

    def PDF(self):
        
        z = (self.s) / (np.sqrt(self.m0))
        Xm = (self.m1 / self.m0) * np.sqrt(self.m2 / self.m4)
        G1 = 2 * (Xm - self.alpha2 ** 2) / (1 + self.alpha2 ** 2)
        R = (self.alpha2 - Xm - G1 ** 2) / (1 - self.alpha2 - G1 + G1 ** 2)
        G2 = (1 - self.alpha2 - G1 + G1 ** 2) / (1 - R)
        G3 = 1 - G1 - G2
        Q = 5 * (self.alpha2 - G3 - G2 * R) / (4 * G1)

        # # Dirlik method according to Ariduru:
        p1 = (G1 / Q) * np.exp(-z / Q)
        p2 = (G2 * z / R ** 2) * np.exp(-z ** 2 / (2 * R ** 2))
        p3 = G3 * z * np.exp(-z ** 2 / 2)
        pnum = p1 + p2 + p3
        pden = np.sqrt(self.m0)
        ps = pnum / pden

        integ = 0
        ds = self.s[1] - self.s[0]
        for i in range(len(self.s)):
            integ += ps[i]*ds
        

        ps = ps/integ

        return ps
    
    def counting_cycles(self):
        ps = self.PDF()
        ds = self.s[1] - self.s[0]
        ns = ps*ds*self.EP*self.xf

        return ns
    
    def loading_spectrum(self):
        Cs = np.zeros(len(self.s))
        ns = self.counting_cycles()

        for i in range(len(self.s)):
            for j in range(i, len(self.s)):
                Cs[i] += ns[j]
        
        return Cs

    def Damage(self):
        
        ps = self.PDF()
        ds = self.s[1] - self.s[0]
        DDK = 0
        for i in range(1,len(ps)):
            DDK += (self.EP*(self.C**(-1))*(self.s[i]**self.k)*ps[i]*ds)

        return DDK

    def Lifes(self):
        TDKs = 1 / self.Damage()
        return TDKs

    def Lifeh(self):
        TDKh = self.Lifes()/(3600)
        return TDKh

    def Life(self):
        TDK = self.Lifes()/self.xf
        return TDK
    
    def relative_error(self, y, x, method="Rainflow", experimental_value=None, type='cycles'):
        if type=="cycles":
            DK_value = self.Life()
            RF_value = Rainflow.rainflowD(self.C, self.k, y, x).Life()
        elif type=="damage":
            DK_value = self.Damage()
            RF_value = Rainflow.rainflowD(self.C, self.k, y, x).Damage()
        elif type!="cycles" and type!="damage":
            raise UnboundLocalError("Invalid type. Try 'cycles' or 'damage'")
        
        if(method == "Rainflow"):
            err = (DK_value - RF_value)/RF_value
        elif(method == "Experimental" and experimental_value != None):
            EX_value = experimental_value
            err = (DK_value - EX_value)/EX_value
        elif(method == "Experimental" and experimental_value == None):
            raise UnboundLocalError("Dexperimental must be different from None for method 'Experimental'")
        elif(method != "Experimental" and method != "Rainflow"):
            raise UnboundLocalError("Invalid Method. Try method='Rainflow' or method='Experimental'")

        return err



import numpy as np
from . import prob_moment, Narrow_Band, Rainflow

class WL:
    def __new__(cls, *args, **kwargs):
        instance = super(WL, cls).__new__(cls)
        return instance

    def __init__(self, k, C, Y, f, xf, s):
        self.k = k
        self.C = C
        self.Y = Y
        self.f = f
        self.xf = xf
        self.s = s
        moment = prob_moment.Probability_Moment(self.Y, self.f)
        self.alpha2 = prob_moment.Probability_Moment(self.Y, self.f).alpha2()
        self.E0 = prob_moment.Probability_Moment(self.Y, self.f).E0()
        self.DNB = Narrow_Band.NB(self.k, self.C, self.Y, self.f, self.xf, self.s).Damage()

    def Damage(self):
        epsylon = np.sqrt(np.abs(1 - self.alpha2 ** 2))

        ak = 0.926 - 0.033 * self.k
        bk = 1.587 * self.k - 2.323

        roWL = ak + (1 - ak) * (1 - epsylon) ** bk

        DWL = roWL * self.DNB

        return DWL

    def Lifes(self):  # ciclos
        TWLs = 1 / self.Damage()
        return TWLs

    def Lifeh(self):  # horas
        TWLh = self.xf * self.Life() / (60 * 60)
        return TWLh
    
    def Life(self):
        TWL = self.Lifes()/self.xf
        return TWL
    
    def relative_error(self, y, x, method="Rainflow", experimental_value=None, type='cycles'):
        if type=="cycles":
            WL_value = self.Life()
            RF_value = Rainflow.rainflowD(self.C, self.k, y, x).Life()
        elif type=="damage":
            WL_value = self.Damage()
            RF_value = Rainflow.rainflowD(self.C, self.k, y, x).Damage()
        elif type!="cycles" and type!="damage":
            raise UnboundLocalError("Invalid type. Try 'cycles' or 'damage'")
        
        if(method == "Rainflow"):
            err = (WL_value - RF_value)/RF_value
        elif(method == "Experimental" and experimental_value != None):
            EX_value = experimental_value
            err = (WL_value - EX_value)/EX_value
        elif(method == "Experimental" and experimental_value == None):
            raise UnboundLocalError("Dexperimental must be different from None for method 'Experimental'")
        elif(method != "Experimental" and method != "Rainflow"):
            raise UnboundLocalError("Invalid Method. Try method='Rainflow' or method='Experimental'")

        return err



import numpy as np
import math
class prob_dist:
    def __new__(cls, *args, **kwargs):
        instance = super(prob_dist, cls).__new__(cls)
        return instance

    def __init__(self, m0, m1, m2, m4, alpha2, s, k, C, EP, xf):
        self.m0 = m0
        self.m1 = m1
        self.m2 = m2
        self.m4 = m4
        self.alpha2 = alpha2
        self.s = s
        self.k = k
        self.C = C
        self.EP = EP
        self.xf = xf

    def PDPeaks(self):
        ratio = (np.sqrt(1-self.alpha2**2)/np.sqrt(2*np.pi*self.m0))
        exp = np.exp(-(self.s**2)/(2*self.m0*(1-self.alpha2**2)))
        ratio2 = (self.alpha2*self.s/self.m0)
        exp2 = np.exp(-(self.s**2)/(2*self.m0))
        #Error Function
        x = (self.alpha2*self.s)/np.sqrt(self.m0*(1-self.alpha2**2))
        for i in range(len(self.s)):
            phi = (1/2)*(math.erf(x[i]/np.sqrt(2))+1)
        
        z = self.s / (np.sqrt(self.m0))
        # # Abaixo contem a função de distribuição normal pelo artigo de Carpinteri
        pp = ratio*exp + ratio2*exp2*phi
        return pp




import numpy as np
from . import prob_moment, Narrow_Band, Rainflow

class AL:
    def __new__(cls, *args, **kwargs):
        instance = super(AL, cls).__new__(cls)
        return instance

    def __init__(self, k, C, Y, f, xf, s):
        self.k = k
        self.C = C
        self.Y = Y
        self.f = f
        self.xf = xf
        self.s = s
        moments = prob_moment.Probability_Moment(self.Y, self.f)
        self.m0 = moments.moment0()
        self.m1 = moments.moment1()
        self.m2 = moments.moment2()
        self.E0 = moments.E0()
        self.alpha075 = moments.alpha075()
        self.alpha1 = moments.alpha1()
        self.alpha2 = moments.alpha2()
        self.EP = moments.EP()
        NB = Narrow_Band.NB(self.k, self.C, self.Y, self.f, self.xf, self.s)
        self.DNB = NB.Damage()
        self.pNB = NB.PDF()

    def PDF(self):

        lambdaAL = self.alpha075**2
        pAL = lambdaAL*self.pNB

        return pAL
    
    def counting_cycles(self):
        pAL = self.PDF()
        ds = self.s[1] - self.s[0]
        nAL = pAL*ds*self.EP*self.xf

        return nAL
    
    def loading_spectrum(self):
        CAL = np.zeros(len(self.s))
        nAL = self.counting_cycles()

        for i in range(len(self.s)):
            for j in range(i, len(self.s)):
                CAL[i] += nAL[j]
        
        return CAL

    def Damage(self):

        DAL = (self.alpha075**2) * self.DNB

        return DAL

    def Lifes(self):  # ciclos
        TALs = 1 / self.Damage()
        return TALs

    def Lifeh(self):  # horas
        TALh = self.xf * self.Life() / (60 * 60)
        return TALh
    
    def Life(self):
        TAL = self.Lifes()/self.xf
        return TAL
    
    def relative_error(self, y, x, method="Rainflow", experimental_value=None, type='cycles'):
        if type=="cycles":
            AL_value = self.Life()
            RF_value = Rainflow.rainflowD(self.C, self.k, y, x).Life()
        elif type=="damage":
            AL_value = self.Damage()
            RF_value = Rainflow.rainflowD(self.C, self.k, y, x).Damage()
        elif type!="cycles" and type!="damage":
            raise UnboundLocalError("Invalid type. Try 'cycles' or 'damage'")
        
        if(method == "Rainflow"):
            err = (AL_value - RF_value)/RF_value
        elif(method == "Experimental" and experimental_value != None):
            EX_value = experimental_value
            err = (AL_value - EX_value)/EX_value
        elif(method == "Experimental" and experimental_value == None):
            raise UnboundLocalError("Dexperimental must be different from None for method 'Experimental'")
        elif(method != "Experimental" and method != "Rainflow"):
            raise UnboundLocalError("Invalid Method. Try method='Rainflow' or method='Experimental'")

        return err


import math
import numpy as np
from . import prob_moment, Rainflow
from scipy.stats import norm

class RC:
    def __new__(cls, *args, **kwargs):
        instance = super(RC, cls).__new__(cls)
        return instance

    def __init__(self, k, C, Y, f, xf, s):
        self.k = k
        self.C = C
        self.xf = xf
        self.Y = Y
        self.f = f
        self.s = s
        moments = prob_moment.Probability_Moment(self.Y, self.f)
        self.m0 = moments.moment0()
        self.m1 = moments.moment1()
        self.m2 = moments.moment2()
        self.E0 = moments.E0()
        self.alpha2 = moments.alpha2()
        self.EP = moments.EP()

    def PDF(self):
        """
        A method to obtain Probability Density Function based in Rice's PDF Process.

        Parameters
        ----------

        """
        
        
        if round(self.alpha2, 4) != 1.0:
            z = self.s/np.sqrt(self.m0)
            ratio = (np.sqrt(1-self.alpha2**2)/np.sqrt(2*np.pi*self.m0))
            exp = np.exp(-(z**2)/(2*(1-self.alpha2**2)))
            
            ratio2 = (self.alpha2*self.s/self.m0)
            exp2 = np.exp(-(z**2)/2)

            #Error Function
            '''
            Considering phi the standard normal distribution and erf the error function,
            the relation between this two functions is given by:
            phi(x) = (1/2)*(1+erf(x/sqrt(2)))
            or
            erf(x) = 2*phi(x*sqrt(2))-1
            '''
            x = self.alpha2*z/np.sqrt(1-self.alpha2**2)
            for i in range(len(self.s)):
                phi = (1/2)*(math.erf(x[i]/np.sqrt(2))+1)

            # # Abaixo contem a função de distribuição normal pelo artigo de Carpinteri
            pRC = ratio*exp + ratio2*exp2*phi
        else:
            z = self.s/np.sqrt(self.m0)
            ratio = (self.s/self.m0)
            exp = np.exp(-(z**2)/2)
        
            pRC = ratio*exp
        
        return pRC
    
    def counting_cycles(self):
        pRC = self.PDF()
        ds = self.s[1] - self.s[0]
        nRC = pRC*ds*self.EP*self.xf

        return nRC
    
    def loading_spectrum(self):
        CRC = np.zeros(len(self.s))
        nRC = self.counting_cycles()

        for i in range(len(self.s)):
            for j in range(i, len(self.s)):
                CRC[i] += nRC[j]
        
        return CRC

    def Damage(self):

        pRC = self.PDF()
        ds = self.s[1] - self.s[0]
        DRC = 0
        for i in range(1,len(pRC)):
            DRC += self.EP*(self.C**(-1))*(self.s[i]**self.k)*pRC[i]*ds

        return DRC
    
    def Lifes(self):
        TRCs = 1/self.Damage()
        return TRCs
    
    def Lifeh(self):
        TRCh = self.Lifes()/3600
        return TRCh

    def Life(self):
        TRC = self.Lifes()/self.xf
        return TRC
    
    def relative_error(self, y, x, method="Rainflow", experimental_value=None, type='cycles'):
        if type=="cycles":
            RC_value = self.Life()
            RF_value = Rainflow.rainflowD(self.C, self.k, y, x).Life()
        elif type=="damage":
            RC_value = self.Damage()
            RF_value = Rainflow.rainflowD(self.C, self.k, y, x).Damage()
        elif type!="cycles" and type!="damage":
            raise UnboundLocalError("Invalid type. Try 'cycles' or 'damage'")
        
        if(method == "Rainflow"):
            err = (RC_value - RF_value)/RF_value
        elif(method == "Experimental" and experimental_value != None):
            EX_value = experimental_value
            err = (RC_value - EX_value)/EX_value
        elif(method == "Experimental" and experimental_value == None):
            raise UnboundLocalError("Dexperimental must be different from None for method 'Experimental'")
        elif(method != "Experimental" and method != "Rainflow"):
            raise UnboundLocalError("Invalid Method. Try method='Rainflow' or method='Experimental'")

        return err



from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()


setup(name='fatiguepy',
      version='1.9.4',
      description='Package to estimate life of random fatigue history with frequency domain methods',
      long_description=long_description,
      long_description_content_type="text/markdown",
      url='https://github.com/thiagonds65/fatiguepy',
      author='Thiago Santos',
      author_email='thiagonds@id.uff.br',
      license='MIT',
      packages=['fatiguepy'])




import math
import numpy as np
from . import prob_moment, Rainflow
from scipy.stats import norm

class NB:
    def __new__(cls, *args, **kwargs):
        instance = super(NB, cls).__new__(cls)
        return instance

    def __init__(self, k, C, Y, f, xf, s):
        self.k = k
        self.C = C
        self.xf = xf
        self.Y = Y
        self.f = f
        self.s = s
        moments = prob_moment.Probability_Moment(self.Y, self.f)
        self.m0 = moments.moment0()
        self.m1 = moments.moment1()
        self.m2 = moments.moment2()
        self.E0 = moments.E0()
        self.alpha2 = moments.alpha2()
        self.EP = moments.EP()

    def PDF(self):
        """
        A method to obtain Probability Density Function for Narrow Band Process.
        ----------
        """

        z = self.s/np.sqrt(self.m0)

        pRL = z*np.exp((-z**2)/2)/np.sqrt(self.m0)

        integ = 0
        ds = self.s[1] - self.s[0]
        for i in range(len(self.s)):
            integ += pRL[i]*ds
        pRL = pRL/integ
        
        return pRL
    
    def counting_cycles(self):
        pRL = self.PDF()
        ds = self.s[1] - self.s[0]
        nRL = pRL*ds*self.EP*self.xf

        return nRL
    
    def loading_spectrum(self):
        CRL = np.zeros(len(self.s))
        nRL = self.counting_cycles()

        for i in range(len(self.s)):
            for j in range(i, len(self.s)):
                CRL[i] += nRL[j]
        
        return CRL

    def Damage(self):
        '''
        O dano calculado pela aproximação da banda estreita pode ser expresso 
        pela seguinte expressão (MRSNIK et. al, 2016)
        EGFkbar2 = math.gamma(1 + self.k / 2)  # Euler Gamma Function

        DNB = self.E0 * (self.C ** (-1)) * (np.sqrt(2 * self.m0)) ** self.k * EGFkbar2 * self.xf
        Mas também pode ser expresso através de uma equação empírica dependente da PDF, 
        como visto abaixo:
        '''
        
        pp = self.PDF()
        ds = self.s[1] - self.s[0]
        DNB = 0
        for i in range(1,len(pp)):
            DNB += self.EP*(self.C**(-1))*(self.s[i]**self.k)*pp[i]*ds
        
        #DNB = self.E0*self.C**(-1)*(2*self.m0)**(self.k/2)*math.gamma(1+self.k/2)

        return DNB
    
    def Lifes(self):
        TNBs = 1/self.Damage()
        return TNBs
    
    def Lifeh(self):
        TNBh = self.Lifes()/3600
        return TNBh

    def Life(self):
        TNB = self.Lifes()/self.xf
        return TNB
    
    def relative_error(self, y, x, method="Rainflow", experimental_value=None, type='cycles'):
        if type=="cycles":
            NB_value = self.Life()
            RF_value = Rainflow.rainflowD(self.C, self.k, y, x).Life()
        elif type=="damage":
            NB_value = self.Damage()
            RF_value = Rainflow.rainflowD(self.C, self.k, y, x).Damage()
        elif type!="cycles" and type!="damage":
            raise UnboundLocalError("Invalid type. Try 'cycles' or 'damage'")
        
        if(method == "Rainflow"):
            err = (NB_value - RF_value)/RF_value
        elif(method == "Experimental" and experimental_value != None):
            EX_value = experimental_value
            err = (NB_value - EX_value)/EX_value
        elif(method == "Experimental" and experimental_value == None):
            raise UnboundLocalError("Dexperimental must be different from None for method 'Experimental'")
        elif(method != "Experimental" and method != "Rainflow"):
            raise UnboundLocalError("Invalid Method. Try method='Rainflow' or method='Experimental'")

        return err



from . import prob_moment, Rainflow, Narrow_Band, Rice, Dirlik, Zhao_Baker, alpha075, Tovo_Benasciutti, Wirsching_Light

__all__ = ['prob_moment', 'Rainflow', 
            'Narrow_Band', 'Wirsching_Light', 
            'Dirlik', 'Zhao_Baker', 'alpha075', 'Tovo_Benasciutti', 'Rice']

## __all__ list tell us what will be import with "from fatiguepy import *"
